#define GBUFFER_MAX_DEPTH 500.0f 

float4 gbufferPack(float depth, float3 diffuse, float3 specular, float fog)
{
	depth = saturate(depth / GBUFFER_MAX_DEPTH);
	
	const float3 bitSh	= float3(255*255, 255, 1);
    const float3 lumVec = float3(0.299, 0.587, 0.114);

	float2 comp;
	comp = depth*float2(255,255*256);
	comp = frac(comp);
	comp = float2(depth,comp.x*256/255) - float2(comp.x, comp.y)/255;
	
	float4 result;
	
	result.r = lerp(1, dot(specular, lumVec), saturate(3 * fog));
	result.g = lerp(0, dot(diffuse, lumVec), saturate(3 * fog));
	result.ba = comp.yx;
	
	return result;
}

float3 lgridPrepareSample(float3 v, float3 n, float4 config0, float4 config1)
{
    // cells are 4 studs in size
    // offset in normal direction to prevent self-occlusion
    // the offset has to be 1.5 cells in order to fully eliminate the influence of the source cell with trilinear filtering
    // (i.e. 1 cell is enough for point filtering, but is not enough for trilinear filtering)
    float3 c = v + n * (1.5f * 4.f);

    return c.xzy * config0.xyz + config1.xyz;
}

float4 lgridSample(sampler3D t, float3 data, float4 config0, float4 config1, float4 border)
{
	float3 pos = saturate(data);

    float3 edge = abs(pos - 0.5);
    float edgef = max(edge.x, max(edge.y, edge.z));

    float4 result = lerp(border, tex3D(t, pos - config1.xyz), step(edgef, 0.49));

	return result;
}
